/*
 * Project name:
    Pressure Click
 * Copyright:
    (c) Mikroelektronika, 2013.
 * Revision History:
    20130410:
      - initial release (VP);
 * Description:
    This program demonstrates measuring of the Pressure and Temprerature.
    Microcontroller take the data about pressure and temperature wich is measured on the Pressure Click.
    It establishes I2C bus communication with  LPS331AP MEMS pressure sensor ( who can messer 260-1260 mbar absolute digital output barometer),
    writes bytes of data on some location, then reads it and displays it on LCD.
 * Test configuration:
    MCU:             PIC18F45K22
                     http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
    Dev.board:       EasyPIC v7
                     http://www.mikroe.com/easypic/
    Oscillator:      HS-Oscillator 8.000 MHz, 4xPLL 32.000 MHz
    Ext. modules:    Pressure Click - ac:Pressure_click
                     http://www.mikroe.com/click/pressure/
                     LCD 2x16 chars ac:LCD
                     http://www.mikroe.com/store/components/displays/
    SW:              mikroC PRO for PIC
                     http://www.mikroe.com/mikroc/pic/

* NOTES:
    - Plug in LCD 2x16 chars on the apropriate place on the Development Board
    - Plug in the Pressure Click on the apropriate place on the Development Board (mikroBUS socket)
    - Turn on BCK line (Back lite on the LCD) at SW4.6
    - Turn on PORTB and PORTD LEDs, turn off PORTA LEDs at SW15.1. (board specific)
*/

#define _REF_P_XL 0x08                       //  Reference pressure (LSB)
#define _REF_P_L 0x09                        //  Reference pressure  (middle)
#define _REF_P_H 0x0A                        //  Reference pressure   (MSB)

#define _WHO_AM_I 0x0F                       //  Device identification

#define _RES_CONF 0x10                       //  Pressure resolution

#define _CTRL_REG1 0x20                      //  Control register 1
#define _CTRL_REG2 0x21                      //  Control register 2
#define _CTRL_REG3 0x22                      //  Control register 3

#define _INT_CFG_REG 0x23                    //  Interrupt configuration
#define _INT_SOURCE_REG 0x24                 //  Interrupt source

#define _THS_P_LOW_REG 0x25                  //  Treshold pressure (LSB)
#define _THS_P_HIGH_REG 0x26                 //  Treshold pressure (MSB)

#define _STATUS_REG 0x27                     //  Status register

#define _PRESS_POUT_XL_REH 0x28                    //  Pressure data (LSB)
#define _PRESS_OUT_L   0x29                  //  Pressure data (middle)
#define _PRESS_OUT_H   0x2A                  //  Pressure data (MSB)

#define _TEMP_OUT_L 0x2B                     //  Temperature data (LSB)
#define _TEMP_OUT_H 0x2C                     //  Temperature data (MSB)

#define _AMP_CTRL 0x30                       //  Analog front ned control


#define _PRESSURE_W_ADDRESS  0xBA            //  BA       //alt 0xB8
#define _PRESSURE_R_ADDRESS  0xBB            //  BB       //alt 0xB9

// Lcd pinout settings
   sbit LCD_RS at LATB4_bit;
   sbit LCD_EN at LATB5_bit;
   sbit LCD_D7 at LATB3_bit;
   sbit LCD_D6 at LATB2_bit;
   sbit LCD_D5 at LATB1_bit;
   sbit LCD_D4 at LATB0_bit;

// Pin direction
   sbit LCD_RS_Direction at TRISB4_bit;
   sbit LCD_EN_Direction at TRISB5_bit;
   sbit LCD_D7_Direction at TRISB3_bit;
   sbit LCD_D6_Direction at TRISB2_bit;
   sbit LCD_D5_Direction at TRISB1_bit;
   sbit LCD_D4_Direction at TRISB0_bit;

char out[14];                                // Declaration of variable
short address, buffer;                       // Declaration of variables
int tempor ;                                 // Declaration of variable
long int p_value;                            // Declaration of variable
float t_value;                               // Declaration of variable


/* Declaration of function, which will initiate I2C communication on relation Miccrocontroler - Pressure Click
  and she will write one byte through I2C, onto memory address on the Pressure Click */

void Pressure_Write(unsigned short address, unsigned short value) {
  I2C1_Start();                              // issue I2C start signal
  I2C1_Wr(_PRESSURE_W_ADDRESS);              // send byte via I2C  (device address + W)
  I2C1_Wr(address);                          // send byte (address of the location)
  I2C1_Wr(value);                            // send data (data to be written)
  I2C1_Stop();                               // issue I2C stop signal
}

// Declaration of function, which will read the measured data from Pressure Click, through I2C communication.

unsigned short Pressure_Read(unsigned short address)  {
  unsigned short tmp = 0;

  I2C1_Start();                              // issue I2C start signal
  I2C1_Wr(_PRESSURE_W_ADDRESS);              // send byte via I2C (device address + W)
  I2C1_Wr(address);                          // send byte (data address)

  I2C1_Repeated_Start();                     // issue I2C signal repeated start
  I2C1_Wr(_PRESSURE_R_ADDRESS);              // send byte (device address + R)
  tmp = I2C1_Rd(_I2C_NACK );                 // Read the data (NO acknowledge)
  I2C1_Stop();                               // issue I2C stop signal

  return tmp;
}

// Declaration of function, which will initiate Pressure values

char Pressure_Init(void)  {
  unsigned short temp;                       // Declaration of local variable
  int err = 0;                               // Declaration of local variable

  Pressure_Write(_RES_CONF, 0x78);           // Initiating the pressure resolution on Pressure Click with writing HEX value in his register ( in this case HEX 0x78)
  temp = Pressure_Read(_RES_CONF);           // Begining of checking of entered value in the register _RES_CONF of the Pressure Click

  if (temp != 0x78) {
    err++;
  }                                          // End of checking of entered value in the register _RES_CONF of the Pressure Click

  Pressure_Write(_CTRL_REG1, 0x74);
  temp = Pressure_Read(_CTRL_REG1);          // Begining of checking of entered value in the register _CTRL_REG1 (Control register 1) of the Pressure Click

  if (temp != 0x74) {
    err++;
  }                                          // End of checking of entered value in the register _CTRL_REG1 (Control register 1) of the Pressure Click

  Pressure_Write(_CTRL_REG1, 0xF4);
  temp = Pressure_Read(_CTRL_REG1);          // Begining of checking of entered value in the register _CTRL_REG1 (Control register 1) of the Pressure Click
  if (temp != 0xF4) {
    err++;
  }                                          // End of checking of entered value in the register _CTRL_REG1 (Control register 1) of the Pressure Click

  temp = Pressure_read(_WHO_AM_I);           // Checking the read-only register from Pressure Click, which contains the device ID (Adresse of Device). For LPS331AP, is set to BBh
  if (temp != 0xBB) {
    err++;
  }

  return err;
}

// Declaration of function which will read the measuring values of pressure from the Pressure Click

long int Pressure_Read_value(void)  {
  unsigned short low_byte;                                   // Declaration of local variable
  unsigned short mid_byte;                                   // Declaration of local variable
  long int Out_pressure;                                     // Declaration of local variable

  Out_pressure = Pressure_Read(_PRESS_OUT_H);                // Taking values of placed setings from registers _PRESS_OUT_H, from Pressure Click
  mid_byte = Pressure_Read(_PRESS_OUT_L);                    // Taking values of placed setings from registers _PRESS_OUT_L, from Pressure Click
  low_byte = Pressure_Read(_PRESS_POUT_XL_REH);              // Taking values of placed setings from registers _PRESS_POUT_XL_REH, from Pressure Click

  Out_pressure = (Out_pressure << 8);                        // Forming the resulting values of pressure
  Out_pressure = (Out_pressure | mid_byte);
  Out_pressure = (Out_pressure << 8);
  Out_pressure = (Out_pressure | low_byte);

  return Out_pressure;
}

// Declaration of function which will read the measuring values of temperature from the Pressure Click

long int Temperature_Read_value(void) {
  unsigned short low_byte; volatile;                         // Declaration of local variable
  int Out_temperature; volatile;                             // Declaration of local variable

  Out_temperature = Pressure_Read(_TEMP_OUT_H);              // Forming the resulting values of temperature
  low_byte = Pressure_Read(_TEMP_OUT_L);

  Out_temperature = (Out_temperature << 8);
  Out_temperature = (Out_temperature | low_byte);

  return Out_temperature;
}

// Declaration of function which will show the values of pressure on the LCDs

void Display_Pressure(long int disp_val)  {
  disp_val >>= 12;
  LongWordToStr(disp_val, out);
  Ltrim(out);
  Lcd_Out(1, 4, out);
  Delay_ms(20);
}

// Declaration of function which will show the values of temperature on the LCDs

void Display_Temperature(float disp_val)  {
  char *res;
  disp_val = ((disp_val/480) + 42.5) ;
  FloatToStr(disp_val, out);
  res = strrchr(out, '.');
  *(res + 2) = 0;
  Lcd_Out(2, 4, out);
  Delay_ms(20);
}

// Begining of the programme code

void main() {
  char err_code;

  ANSELA = 0;                                        // Configure AN pins as digital
  ANSELB = 0;                                        // Configure AN pins as digital
  ANSELC = 0;                                        // Configure AN pins as digital
  ANSELD = 0;                                        // Configure AN pins as digital
  ANSELE = 0;                                        // Configure AN pins as digital
  SLRCON = 0;                                        // Configure SLEW RATE CONTROL REGISTER
  C1ON_bit = 0;                                      // Disable comparators
  C2ON_bit = 0;

  I2C1_Init(100000);                                 // Initializes I2C with desired clock
  Pressure_Init();                                   // Initializes pressure
  Delay_ms(10);                                      // Placing the delay

  Lcd_Init();                                        // Initializes LCD
  Lcd_Cmd(_LCD_CLEAR);                               // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);                          // Cursor off
  Lcd_Out(1,1,"Pressure init");                      // Write text in first row

  Delay_ms(321);                                     // Placing the delay

  err_code = Pressure_Init();

  if ( err_code == 0)        {                       // Writing the text into first row of LCD
    Lcd_out(2, 1, "Init OK");
  }
  else        {
    Lcd_Out(2, 1, "Init error!");
    Lcd_Chr(2, 11, (48 + err_code));
  }

  Delay_ms(500);                                     // Placing the delay
  Delay_ms(1000);                                    // Placing the delay

  Lcd_Cmd(_LCD_CLEAR);                               // Clear display
  Lcd_Out(1, 1, "P: ");
  Lcd_Out(2, 1, "T: ");

  while(1)          {                                // Writing the exact values of pressure and temperature on the display of LCD
    Lcd_Out(1, 3, "          ");
    p_value = Pressure_Read_value();
    Display_Pressure(p_value);

    t_value = (float)Temperature_Read_value();
    Display_Temperature(t_value);

    Delay_ms(700);                                   // Placing the delay
  }
}